; some general utilities for wined
; v. 1.05	2003 Jun 06 09:25:54
;  W. Lenerz  1994 - 2018

	section gu

	xdef	make1to9
	xdef	read_sec
	xdef	readsec2
	xdef	set_posn
	xdef	setpos2
	xdef	setpos3
	xdef	input_str
	xdef	nextsec
	xdef	calc_sec
	xdef	rest_old
	xdef	backsec
	xdef	set_blocks
	xdef	set_undl

	xref	do_rstr
	xref	srch	; title for input string
	xref	hex	; prompt
	xref	colw	; colourways
	xref	wintype ; in wined_cfg

	xref	long_cha
	xref	small_cha
	xref	vsmal_cha


	include dev1_asm_wined_includes
	include dev8_keys_wwork
	include DEV8_keys_qdos_io

;---------------------------------------------------------------
; this makes a hex number out of a single valid hex digit in d1
; all regs preserved except d1
; no condition codes
;---------------------------------------------------------------

make1to9
	cmpi.b	#'9',d1
	bgt.s	mk_ok
	subi.b	#'0',d1
	rts
mk_ok	subi.b	#$37,d1
	rts


;---------------------------------------------------------------
; this reads one sector
; entry 			exit
; d0				0 or error
; d1				corr
; d2				corr
; d3				corr
; a0				screen channel ID
; a1				pointer to data for sector
;
; condition codes set on exit
;---------------------------------------------------------------
readsec2
	bsr.s	setpos2
	bra.s	readcom
read_sec
	bsr.s	set_posn		; set file position first (this also
readcom bne.s	read_out		; sets d3 to -1), and a0 to file_id
	move.w	seclen(a4),d2		; number of bytes per sector
	lea	file_sp(a4),a1
	moveq	#iob.fmul,d0		; get 512 or 2k bytes now
	trap	#3
	lea	file_sp(a4),a1
	move.l	ww_chid(a4),a0		; reset screen channel id
	tst.l	d0			; ok?
	bne.s	read_err		; no!
	move.w	#0,endofile(a4) 	; always make sure of that!
read_out
	rts
read_err
	cmp.l	#-10,d0 		; was it end of file?
	bne.s	read_out		; no->
	add.l	d1,a1			; yes, set others to 0 point to last byte
	move.w	d1,endofile(a4)
	move.w	#511,d0
	sub.w	d1,d0
floop
	move.b	#0,(a1)+
	dbf	d0,floop
	moveq	#0,d0			; simulate no error
	bra.s	read_out


; this sets the file position
; d1					new file position
; a0					file channel ID
; A1					corr

; returns 0 or error
; condition codes set

set_posn
	move.l	sector(a4),d1
setpos2 moveq	#-1,d3
	add.l	partit(a4),d1		; add possible offset
setpos3 move.l	file_id(a4),a0
	moveq	#iof.posa,d0		; now set to this position
	trap	#$3
	tst.l	d0
	rts


;---------------------------------------------------------------
; this inputs a string
; entry:				exit
; d0					error or 0
; a0: points to title			corr
; a1: points to place for string	same
; this will also be default
; a3: points to prompt			corr
; all other regs preserved, condition codes set on exit
;---------------------------------------------------------------

ipregs	reg	d1-d3/a2

input_str
	movem.l ipregs,-(a7)
	moveq	#66,d1		; max length of string
	move.l	#winpos,d2
	move.w	#4,d2
	lea	colw,a2
	moveq	#0,d3
	move.b	(a2),d3
	move.l	a1,a2		; return string is default
	jsr	do_rstr
	movem.l (a7)+,ipregs
	tst.l	d0
	rts
;-------------------------------------------------------------
;
; this gets the next sector data into d1, i.e. the file 'position' for
; the next sector
; on exit, d0 is not set as error condition. condition codes are not set
; on exit, d0 = 0 if the end of the disk was reached
; else d0.h<>0

secregs reg	a1/d2/d3
nextsec movem.l secregs,-(a7)
	move.l	sector(a4),d1
	move.l	d1,d4		; keep old sector number
	moveq	#-1,d0		; this is for search: d0 = 0 if end of disk
	cmp.w	#flodisk,type(a4)	; are we editing a floppy?
	beq.s	is_floppy	; yes->...
others	cmp.w	#winedit,type(a4)	; are we editing a win disk?
	bne.s	ot2		; no, so must be file->...
	lea	wintype,a1	; point to type of win (miracle/qubide)
	tst.b	(a1)		; is win a qubide?
	bne.s	is_qubide	; yes->...
ot2	add.l	step(a4),d1	; next sector etc...
	cmp.l	endsek(a4),d1	; did we overshoot end of disk?
	bls.s	ot3		; no->...
	moveq	#0,d1
	moveq	#0,d0
ot3	movem.l (a7)+,secregs
	rts

is_qubide moveq #0,d3		; lowest sector number for qubide
	bra.s	fl_qu_com
is_floppy
	moveq #1,d3	      ; lowest sector number for floppies
fl_qu_com
	lea   more_sp(a4),a1
	moveq	#-1,d2
	move.l	side(a4),(a1)+	; stock the old values somewhere
	move.w	fl_sect(a4),d1
	move.w	d1,(a1) 	; sector
	addq.w	#1,d1		; new sector
	cmp.w	maxsect(a4),d1	; must be no more than max number
	ble.s	sect_ok2		; ok?  yes, no need to calc trk,side ->...
	move.w	d3,d1		; else, new sector number; ...
	move.w	side(a4),d0	; ... and we must now increase the ...
	addq.w	#1,d0		; .. head (side) number, but do not..
	cmp.w	maxsides(a4),d0 ; exceed max number of heads/sides
	ble.s	side_ok 	; ok? -> yes->...
	moveq	#0,d0		; else set side/head to 0
	move.w	track(a4),d2	; number of tracks
	addq.w	#1,d2		; +1
	cmp.w	maxtrack(a4),d2 ; but we mustn't exceed this!
	ble.s	set_trk
	moveq	#0,d2		; else we get back to start of disk
set_trk move.w	d2,track(a4)
side_ok move.w	d0,side(a4)	; new head number
sect_ok2
	move.w	d1,fl_sect(a4)	; new sector number
	bsr.s	calc_sec		; now calculate new sector, into d1
	move.l	d2,d0		; show when end of disk reached
	movem.l (a7)+,secregs
	rts
;----------------------------------------------------------------------
; this puts the true sector data into d1, for floppy reads
; on exit d1 = sector
; all other regs kept
; condition codes not set
;---------------------------------------------------------------

calc_sec
	move.w	track(a4),d1
	swap	d1		; track * 65536 (i.e. in upper word)
	move.w	side(a4),d1
	lsl.w	#8,d1		; side into upper byte of lower word
	add.w	fl_sect(a4),d1	; sector into lower bytes of lower word
	rts


;-------------------------------------------------------------
; this gets the last sector data into d1
;---------------------------------------------------------------
backsec movem.l secregs,-(a7)
	move.l	sector(a4),d1
	move.l	d1,d4		; keep old sector number
	cmp.w	#flodisk,type(a4) ; are we editing a floppy?
	beq.s	is_disk2	; yes->...
	cmp.w	#winedit,type(a4)	; are we editing a win disk?
	bne.s	others3 	; no, so must be file->...
	lea	wintype,a1	; point to type of win (miracle/qubide)
	tst.b	(a1)		; is win a qubide?
	bne.s	is_qubid2	; yes->...
others3 movem.l (a7)+,secregs
	sub.l	step(a4),d1	; new sector
	bmi.s	bsout2		; if <0,  we go to last sector
bsout	rts
bsout2	move.l	endsek(a4),d1	; here
	rts

is_qubid2
	moveq	#0,d3		; lowest sector number for floppies
	bra.s	fl_qu_co2

is_disk2
	moveq	#1,d3		; lowest sector number for floppies
fl_qu_co2
	lea	more_sp(a4),a1
	move.l	side(a4),(a1)+	; stock the old values somewhere
	move.w	fl_sect(a4),d1
	move.w	d1,(a1)
	beq.s	ls		; if this is 0, then it is low sector
	subq.b	#1,d1		; new sector value
	cmp.w	d3,d1		; must be no more than max number
	bge.s	sect_ok2		; ok?  yes ->...
ls	move.w	maxsect(a4),d1	; else this is new sector

	move.w	side(a4),d0	; side/heads
	beq.s	ls2
	subq.w	#1,d0		; decrease the head/side number, but...
	bge	side_ok 	; ...do not give negative heads
ls2	move.w	maxsides(a4),d0 ; else set side/head to max
	move.w	track(a4),d2	; number of tracks
	beq.s	ls3
	subq.w	#1,d2		; -1
	bge	set_trk
ls3	move.w	maxtrack(a4),d2
	bra	set_trk


;------------------------------------------------------------
; this restores old side, track & sector if pb
;---------------------------------------------------------------
rest_old
	move.l	a1,-(a7)
	lea	side(a4),a1
	move.l	more_sp(a4),(a1)+	   ; stock the old values somewhere
	move.w	more_sp+4(a4),(a1)
	move.l	(a7)+,a1
others2 move.l	d4,d1
	rts


;---------------------------------------------------------------
; this sets the max number of blocks to be seen in window
; entry:				exit
; all regs preserved, condition codes not set on exit
;---------------------------------------------------------------
sbregs	reg	d0/a1/a2
set_blocks
	movem.l sbregs,-(a7)
	cmp.w	#512,seclen(a4) 	; do we have a normal "512" bytes sector?
	beq.s	is_normal		; yes->...

is_big	moveq	#1,d0			; item available
	cmp.w	#$f,nr_rows(a4) 	; small window?
	beq.s	big_smal		; yes->...
big_big 				; big sector,big window
	move.w	#3,winsects(a4) 	; no, so there are 4 blocks
	lea	small_cha,a2
	bra.s	sb_all
big_smal				; big sector, small window
	move.w	#7,winsects(a4) 	; there are 8 blocks
	lea	long_cha,a2
	bra.s	sb_all

; here we have a normal sized block (512 bytes)
is_normal
	cmp.w	#$f,nr_rows(a4) 	; small window?
	beq.s	smal_smal		; yes->...
smal_big				; here: smll secor, big wdw
	move.w	#0,winsects(a4) 	; 1 subdivision only
	lea	vsmal_cha,a2		; text for this
	move.b	#$11,d0 		; item unavailable
	bra.s	sb_all
smal_smal
	move.w #1,winsects(a4)
	lea	vsmal_cha,a2
	move.b	#$1,d0


sb_all	move.l	ww_plitm(a4),a1 	; point to items
	add.l	#it_1*wwl.elen,a1	; point to this item
	move.l	a2,wwl_pobj(a1) 	; new text for item
	move.l	(a4),a1
	move.b	d0,it_1+$40(a1) 	; new status of this item
	movem.l (a7)+,sbregs
	rts

;---------------------------------------------------------------
; this sets underline for the 1->8 items,according to the row number
; entry:				exit
; all regs preserved, condition codes not set on exit
;---------------------------------------------------------------
undlregs reg	d0/a2
set_undl
	movem.l undlregs,-(a7)
	move.l	ww_plitm(a4),a2 ; point to menu items list
	add.l	#it_1*wwl.elen,a2
	move.w	row(a4),d0
	lsl.w	#1,d0		; *2
	addq.w	#1,d0
	neg.w	d0
	move.b	d0,wwl_type(a2) ; new type (i.e. underlining)
	move.l	(a4),a2 	; point to status area
	bset	#0,it_1+$40(a2) ; set redraw bit
	movem.l (a7)+,undlregs
	rts

	end
